@@ -1,7 +1,7 @@
 %
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -625,7 +625,11 @@ signature_scheme(?ECDSA_SHA1) -> ecdsa_sha1;
 signature_scheme(SignAlgo) when is_integer(SignAlgo) ->
     <<?BYTE(Hash),?BYTE(Sign)>> = <<?UINT16(SignAlgo)>>,
     try
-        {ssl_cipher:hash_algorithm(Hash), ssl_cipher:sign_algorithm(Sign)}
+        case {hash_algorithm(Hash), sign_algorithm(Sign)} of
+            {unassigned, _} -> unassigned;
+            {_, unassigned} -> unassigned;
+            Scheme -> Scheme
+        end
     catch
         _:_ ->
             unassigned
