@@ -242,11 +242,11 @@ hello(#client_hello{client_version = _ClientVersion,
                     extensions = #{client_hello_versions :=
                                        #client_hello_versions{versions = ClientVersions}
                                   }} = Hello,
-      #{versions := Versions} = SslOpts,
-      Info, Renegotiation) ->
+      #{versions := Versions = [Version |_]} = SslOpts,
+      Info, Renegotiation) when ?TLS_GTE(Version, ?TLS_1_2)->
     try
-        Version = ssl_handshake:select_supported_version(ClientVersions, Versions),
-        do_hello(Version, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation)
+        SelectedVersion = ssl_handshake:select_supported_version(ClientVersions, Versions),
+        do_hello(SelectedVersion, Versions, CipherSuites, Hello, SslOpts, Info, Renegotiation)
     catch
 	error:Reason:ST ->
             ?SSL_LOG(info, handshake_error, [{reason,Reason}, {stacktrace, ST}]),
@@ -336,13 +336,13 @@ handle_client_hello(Version,
                     Renegotiation) ->
     case tls_record:is_acceptable_version(Version, Versions) of
 	true ->
-            SupportedHashSigns =
-                ssl_handshake:supported_hashsigns(maps:get(signature_algs, SslOpts, undefined)),
+            SigAlgs = ssl_handshake:supported_hashsigns(maps:get(signature_algs, SslOpts, undefined)),
+            SigAlgsCert = signature_algs_cert(Version, SslOpts, SigAlgs),
             Curves = maps:get(elliptic_curves, HelloExt, undefined),
             ClientHashSigns = get_signature_ext(signature_algs, HelloExt, Version),
             ClientSignatureSchemes = get_signature_ext(signature_algs_cert, HelloExt, Version),
 	    AvailableHashSigns = ssl_handshake:available_signature_algs(
-				   ClientHashSigns, SupportedHashSigns, Version),
+				   ClientHashSigns, SigAlgs, Version),
 	    ECCCurve = ssl_handshake:select_curve(Curves, SupportedECCs, ECCOrder),
 	    {Type, #session{cipher_suite = CipherSuite,
                             own_certificates = [OwnCert |_]} = Session1}
@@ -357,7 +357,7 @@ handle_client_hello(Version,
 		    #{key_exchange := KeyExAlg} = ssl_cipher_format:suite_bin_to_map(CipherSuite),
                     case ssl_handshake:select_hashsign({ClientHashSigns, ClientSignatureSchemes},
                                                        OwnCert, KeyExAlg,
-                                                       SupportedHashSigns,
+                                                       SigAlgsCert,
                                                        Version) of
 			#alert{} = Alert ->
 			    throw(Alert);
@@ -373,6 +373,16 @@ handle_client_hello(Version,
 	    throw(?ALERT_REC(?FATAL, ?PROTOCOL_VERSION))
     end.
 
+signature_algs_cert(Version, SslOpts, SigAlgs)  when ?TLS_GTE(Version, ?TLS_1_2) ->
+    case maps:get(signature_algs_cert, SslOpts, undefined) of
+        undefined ->
+            SigAlgs;
+        SigAlgsCert ->
+            ssl_handshake:supported_hashsigns(SigAlgsCert)
+    end;
+signature_algs_cert(_,_,_) ->
+    undefined.
+
 handle_client_hello_extensions(Version, Type, Random, CipherSuites,
                                HelloExt, SslOpts, Session0, ConnectionStates0, 
                                Renegotiation, HashSign) ->
